package com.yihu.wlyy.statistics.job.business;

import com.yihu.wlyy.statistics.dao.*;
import com.yihu.wlyy.statistics.etl.model.ETLModel;
import com.yihu.wlyy.statistics.etl.role.Level1Role;
import com.yihu.wlyy.statistics.etl.storage.DBStorage;
import com.yihu.wlyy.statistics.model.hosptial.Hospital;
import com.yihu.wlyy.statistics.model.job.QuartzJobLog;
import com.yihu.wlyy.statistics.model.job.WlyyQuotaResult;
import com.yihu.wlyy.statistics.model.signfamily.SignFamily;
import com.yihu.wlyy.statistics.model.system.Town;
import com.yihu.wlyy.statistics.model.team.AdminTeam;
import com.yihu.wlyy.statistics.vo.WlyyJobConfigVO;
import com.yihu.wlyy.statistics.vo.WlyyQuotaVO;
import org.quartz.Job;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.context.support.SpringBeanAutowiringSupport;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by Administrator on 2016.10.10.
 * 每天的签约到达量统计
 */
@Component
public class AllSignJob implements Job {
    private WlyyQuotaVO wlyyQuota;//指标对象
    private WlyyJobConfigVO wlyyJobConfig;//配置对象
    @Autowired
    private QuartzJobLogDao quartzJobLogDao;//执行日志Dao
    @Autowired
    private SignFamilyDao signFamilyDao;
    @Autowired
    private HospitalDao hospitalDao;
    @Autowired
    private TownDao townDao;
    @Autowired
    private JdbcTemplate jdbcTemplate;
    @Autowired
    private DoctorAdminTeamDao doctorAdminTeamDao;
    @Autowired
    private Level1Role levelRole;
    @Autowired
    private DBStorage dbStorage;

    String date; //起始时间

    public void execute(JobExecutionContext context)
            throws JobExecutionException {
        try{
            SpringBeanAutowiringSupport.processInjectionBasedOnCurrentContext(this);
            //初始化参数
            JobDataMap map = context.getJobDetail().getJobDataMap();
            wlyyQuota = (WlyyQuotaVO) map.get("quota");
            wlyyJobConfig = (WlyyJobConfigVO) map.get("jobConfig");

            date= StringUtils.isEmpty(map.get("yesterday"))?SignJob.getDayString(-1):map.get("yesterday").toString();
            //计算指标
            computequota();
        }catch (Exception e){
            //如果出錯立即重新執行
            JobExecutionException e2 =new JobExecutionException(e);
            e2.setRefireImmediately(true);
        }
    }

    /**
     * 计算指标
     */
    private void computequota() {
        deleteData();
        //新建任务日志对象
        QuartzJobLog quartzJobLog = new QuartzJobLog();
        quartzJobLog.setJobStartTime(new Date());
        quartzJobLog.setJobId(wlyyJobConfig.getId());
        quartzJobLog.setJobName(wlyyJobConfig.getJobName());

        //查找出系统全部的全科医生
        List<AdminTeam> adminTeams=doctorAdminTeamDao.findAllTeam();
        Map<String, AdminTeam> adminTeamMap = new HashMap<String, AdminTeam>();
        for (AdminTeam adminTeam : adminTeams) {
            adminTeamMap.put(adminTeam.getId()+"", adminTeam);
        }
        //查找出系统全部的机构
        List<Hospital> hospitals = hospitalDao.findHospital2();
        Map<String, Hospital> hospitalsMap = new HashMap<String, Hospital>();
        for (Hospital hospital : hospitals) {
            hospitalsMap.put(hospital.getCode(), hospital);
        }
        //查找出厦门市全部的区
        List<Town> towns = townDao.findByCityCode(Constant.city);
        Map<String, Town> townsMap = new HashMap<String, Town>();
        for (Town town : towns) {
            townsMap.put(town.getCode(), town);
        }
        int page=1;
        int start=1;
        int pageSize=10000;
        int lastSize=0;
        String dateTemp = date + Constant.quota_date_last;
        Long cityCount = new Long(0L); //统计到市的数目
        Long townCount = new Long(0L); //统计到所有区的数目
        Long orgCount = new Long(0L);//统计到所有机构的数目
        Long adminCount = new Long(0L);//统计到所有团队的数目
        Long errorCount = new Long(0L);//脏数据
        Boolean isAll = true;//是否统计失败
        StringBuffer errorContent = new StringBuffer();
        //统计有已改簽的
        List<ETLModel> etlModels = new ArrayList<ETLModel>();
        List<SignFamily> signFamilys =null;
        while (true){
            String sql=" select id,code,idcard,hospital,admin_team_code,expenses_status from wlyy_sign_family a where  a.type =2 and a.status >= 1  and expenses_status=1 and a.apply_date<= '"+dateTemp+"' limit "+start+","+pageSize ;
            signFamilys = jdbcTemplate.query(sql,new BeanPropertyRowMapper(SignFamily .class));  // signFamilyDao.findByJiatingSignYesterday(dateTemp);
            //数据过滤清洗出脏数据  -----------start
            for (SignFamily signFamily : signFamilys) {
                ETLModel etlModel = new ETLModel();
                String orgCode = signFamily.getHospital();
                if (StringUtils.isEmpty(orgCode)) {
                    errorContent.append("签约code：" + signFamily.getCode() + ",签约表中(wlyy_sign_family)中的hospital数据为空");
                    isAll = false;
                    errorCount++;
                    continue;
                }
                String orgCodeTemp = "";
                if (!"00".equals(orgCode.substring(orgCode.length() - 2, orgCode.length()))) {
                    //站
                    orgCodeTemp = orgCode.substring(0, orgCode.length() - 2) + "00";
                } else {
                    //社区
                    orgCodeTemp = orgCode;
                }
                //判断社区有没有值
                Hospital hospital = hospitalsMap.get(orgCodeTemp);
                if (hospital == null) {
                    errorContent.append("签约code：" + signFamily.getCode() + ",签约表中(wlyy_sign_family)中的hospital数据不存在");
                    isAll = false;
                    errorCount++;
                    continue;
                }
                Long adminId = signFamily.getAdminTeamCode();
                if (adminId == null || adminId <= 0) {
                    errorContent.append("签约code：" + signFamily.getCode() + ",签约表中(wlyy_sign_family)中的Admin_Team_Id数据为空");
                    isAll = false;
                    errorCount++;
                    continue;
                }
                AdminTeam adminTeam = adminTeamMap.get(signFamily.getAdminTeamCode() + "");
                if (adminTeam == null) {
                    errorContent.append("签约code：" + signFamily.getCode() + ",签约表中(wlyy_sign_family)中的Admin_Team_Id数据不存在");
                    isAll = false;
                    errorCount++;
                    continue;
                }
                String idCard = signFamily.getIdcard();
                if (idCard == null || (idCard.length() != 18 && idCard.length() != 15)) {
                    errorContent.append("签约code：" + signFamily.getCode() + ",签约表中(wlyy_sign_family)中的idCard数据异常");
                    isAll = false;
                    errorCount++;
                    continue;
                }
                String town = hospital.getTown();
                if (StringUtils.isEmpty(town)) {
                    errorContent.append("签约code：" + signFamily.getCode() + ",机构表的中的town为空");
                    isAll = false;
                    errorCount++;
                    continue;
                }
                Town townObj = townsMap.get(town);
                if (townObj == null) {
                    errorContent.append("签约code：" + signFamily.getCode() + ",机构表的中的town不存在");
                    isAll = false;
                    errorCount++;
                    continue;
                }
                etlModel.setLevel2Key(signFamily.getExpensesStatus());//设置二级维度的Key
                etlModel.setAdminTeam(adminTeam.getId() + "");
                etlModel.setHospital(orgCodeTemp);
                etlModel.setTown(hospital.getTown());
                etlModel.setCity(Constant.city);
                etlModel.setIdcard(signFamily.getIdcard());
                etlModels.add(etlModel);
                //统计数目+1
                cityCount++;
                townCount++;
                orgCount++;
                adminCount++;
            }
            if(signFamilys.size()!=pageSize){
                lastSize=signFamilys.size();
                signFamilys.clear();
                break;
            }else{
                start =page*pageSize+1;
                page++;
                signFamilys.clear();
            }
        }
        //数据过滤清洗出脏数据  -----------end
        List<WlyyQuotaResult> saveData=null;
        try{
            //统计数据
            List<Map<String, List<ETLModel>>>  returnData= levelRole.elt(etlModels);
            //保存数据
            dbStorage.saveByLevel1(returnData,date,wlyyQuota);
        }catch (Exception e){
            e.printStackTrace();
            errorContent.append("统计失败:"+e.getMessage());
        }
        //保存日志
        quartzJobLog.setJobEndTime(new Date());
        quartzJobLog.setJobContent(saveContent((page-1)*pageSize+lastSize,adminCount,orgCount,townCount,cityCount,isAll,errorContent,errorCount));
        quartzJobLog.setJobType(isAll?"1":"0");
        dbStorage.saveLog(quartzJobLog);
    }

    @Transactional
    private void deleteData() {
        jdbcTemplate.execute("delete from wlyy_quota_result where quota_date='"+date+"' and quato_code='"+13+"'");
    }

    private String saveContent(Integer count, Long qkCount, Long orgCount, Long townCount, Long cityCount, boolean isAll, StringBuffer errorContent,Long errorCount) {
        StringBuffer string=new StringBuffer("统计"+date+" 的簽約数据完成 ,数据库查询到簽約数据:"+count);
        string.append(",过滤的脏数据数目:"+errorCount);
        string.append(",统计到市的数据总数:"+cityCount);
        string.append(",统计到区的数据总数:"+townCount);
        string.append(",统计到机构的数据总数:"+orgCount);
        string.append(",统计到团队的数据总数:"+qkCount);
        string.append("，是否统计成功:"+isAll);
        if(!isAll){
          //  string.append(",失败原因:"+errorContent);
        }
        return string.toString();
    }

    public String getYesterday() {
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -1);
        String yesterday = new SimpleDateFormat("yyyy-MM-dd").format(cal.getTime());
        return yesterday;
    }

}
